/*
 * Copyright (c) 2003 Shaven Puppy Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'Shaven Puppy' nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package riven.classpath;

import java.io.*;
import java.util.*;

public class Pair<A, B> implements Serializable {
	@SafeVarargs
	public static <K, V> Map<K, V> asMap(Pair<K, V>... pairs) {
		return asMap(Arrays.asList(pairs));
	}

	public static <K, V> Map<K, V> asMap(List<Pair<K, V>> pairs) {
		Map<K, V> map = new HashMap<>();
		for (Pair<K, V> pair : pairs) {
			if (map.put(pair.first(), pair.second()) != null) {
				throw new IllegalStateException("duplicate key: " + pair.first());
			}
		}
		return map;
	}

	private static final long serialVersionUID = 1794164589423525789L;

	private final A a;
	private final B b;

	public Pair(A a, B b) {
		this.a = a;
		this.b = b;
	}

	public A first() {
		return this.a;
	}

	public B second() {
		return this.b;
	}

	@Override
	public int hashCode() {
		int ah = a == null ? 0 : a.hashCode();
		int bh = b == null ? 0 : b.hashCode();
		return ah ^ (bh * 37);
	}

	@Override
	public String toString() {
		return "Pair[" + this.a + ", " + this.b + "]";
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Pair<?, ?>)) {
			return false;
		}

		Pair<?, ?> that = (Pair<?, ?>) obj;
		return eq(this.a, that.a) && eq(this.b, that.b);
	}

	private static final boolean eq(Object a, Object b) {
		if (a == b) {
			return true;
		}
		if (a == null ^ b == null) {
			return false;
		}

		if (a != null) {
			return a.equals(b);
		}
		if (b != null) {
			return b.equals(a);
		}

		return false;
	}
}
