/*
 * Copyright (c) 2003 Shaven Puppy Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'Shaven Puppy' nor the names of its contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package riven.classpath;

import java.io.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.*;

public class Streams {

	private static final long worker_thread_stacksize = 256 * 1024; // FIXME
	private static final AtomicLong worker_thread_counter = new AtomicLong();
	private static final ThreadGroup streams_group = new ThreadGroup("Streams");
	private static final ThreadFactory thread_factory = new ThreadFactory() {
		@Override
		public Thread newThread(Runnable task) {
			String name = "io-worker-" + worker_thread_counter.incrementAndGet();
			Thread thread = new Thread(streams_group, task, name, worker_thread_stacksize);
			Log.info("Streams: " + name);
			return thread;
		}
	};

	public static ThreadFactory getStreamThreadFactory() {
		return thread_factory;
	}

	public static void spawn(Runnable task) {
		thread_factory.newThread(task).start();
	}

	//

	public static byte[] loadResource(String path) {
		try {
			return Streams.readFully(Thread.currentThread().getContextClassLoader().getResourceAsStream(path));
		} catch (Exception exc) {
			throw new IllegalStateException(exc);
		}
	}

	public static String loadResourceAsString(String path) {
		try {
			return Text.utf8(loadResource(path));
		} catch (Exception exc) {
			throw new IllegalStateException(exc);
		}
	}

	//

	public static final void writeFile(File file, byte[] data) throws IOException {
		try (OutputStream out = new FileOutputStream(file)) {
			out.write(data);
		}
	}

	public static final byte[] readFile(File file) throws IOException {
		return readFully(new FileInputStream(file));
	}

	//

	public static final byte[] readFully(InputStream in) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		copy(in, out);
		return out.toByteArray();
	}

	//

	public static final void async(final InputStream input, final OutputStream output) {
		new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					copy(input, output);
				} catch (IOException exc) {
					exc.printStackTrace();
				}
			}
		}).start();
	}

	//

	public static final long copy(InputStream input, OutputStream output) throws IOException {
		return copy(input, output, false);
	}

	public static final long copy(InputStream input, OutputStream output, boolean consumeReadException) throws IOException {
		byte[] buffer = new byte[4096];

		long copied = 0;

		while (true) {
			int got;
			try {
				got = input.read(buffer);
			} catch (IOException exc) {
				if (!consumeReadException) {
					throw exc;
				}
				got = -1;
			}
			if (got == -1) {
				break;
			}

			output.write(buffer, 0, got);
			copied += got;
		}

		// never consume these IOExceptions, they indicate corrupt state
		try {
			input.close();
		} finally {
			output.close();
		}

		return copied;
	}

	public static void close(Closeable c) {
		close(c, false);
	}

	public static void close(Closeable c, boolean printExc) {
		if (c == null) {
			return;
		}

		try {
			c.close();
		} catch (IOException exc) {
			if (printExc) {
				exc.printStackTrace();
			}
		}
	}

	public static String stdinAsk(String q) {
		System.out.print(q);
		System.out.print(": \r\n");
		System.out.flush();
		return stdinReadLine();
	}

	public static String stdinReadLine() {
		try {
			StringBuilder sb = new StringBuilder();
			while (true) {
				int b = System.in.read();
				if (b == -1) {
					break;
				}
				char c = (char) b;
				if (c == '\r' || c == '\n') {
					if (sb.length() == 0) {
						continue;
					}
					break;
				}
				sb.append(c);
			}
			return sb.toString();
		} catch (IOException exc) {
			throw new IllegalStateException(exc);
		}
	}
}
